<?php

// No direct access.
defined('_JEXEC') or die;

class WrongRefTypeException extends Exception
{}

class AssociatedObjectUnsavedException extends Exception
{}

class CommonClubsModelColumnRef extends AbstractCommonClubsModelColumn
{
    
    protected $className;
    
    public function __construct($alias, $className, $column, $filter)
    {
        parent::__construct($alias, $filter, $column);
        
        if(empty($className))
            throw new Exception('Classname must be non-empty.');
        
        $this->className = $className;
    }
    
    public function isSimpleType()
    {
        return false;
    }
    
    public function getQuotedValue($q, $value)
    {
        if(is_null($value))
            return 'NULL';
        else
        {
            if(! ( $value instanceof $this->className ) )
                throw new WrongRefTypeException();
            
            $id = $value->getId();
            if($id === 'new')
                throw new AssociatedObjectUnsavedException();
            
            return $id;
        }
    }
    
    public function packValue($value)
    {
        if(is_null($value))
            return null;
        
        if(! ($value instanceof $this->className) )
            throw new WrongRefTypeException();
        
        $id = $value->getId();
        
        if($id === 'new')
            throw new AssociatedObjectUnsavedException();
        
        return (int) $id;
    }
    
    public function unpackValue($value)
    {
        if(empty($value))
            return null;
        
        if(is_string($value) && preg_match('/^[0-9]+$/', $value))
            $value = (int) $value;
            
        if(! is_int($value))
            throw new Exception('Reference with non-integer value');
            
        $factoryName = $this->getFactoryNameOfClass($this->className); // XXX Use attribute?
        $factory = new $factoryName();
        return $factory->loadById($value);
    }
    
    private const CLASSNAME_MAP = array(
        
    );
    
    /**
     * @todo This must be done better and more cleanly
     * @param string $className
     * @return AbstractCommonClubsModelFactory
     */
    private function getFactoryNameOfClass($className)
    {
        if(empty(self::CLASSNAME_MAP[$className]))
        {
            $parts = array();
            if(preg_match('/^CommonClubsModel(.*)/', $className, $parts))
            {
                return "CommonClubsModelFactory{$parts[1]}";
            }
            throw new Exception("Unknown mapping of class $className");
        }
        
        return self::CLASSNAME_MAP[$className];
    }
    
}
